/*  RetroArch - A frontend for libretro.
 *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
 *  Copyright (C) 2011-2016 - Daniel De Matteis
 *  Copyright (C) 2014-2016 - Jean-André Santoni
 *
 *  RetroArch is free software: you can redistribute it and/or modify it under the terms
 *  of the GNU General Public License as published by the Free Software Found-
 *  ation, either version 3 of the License, or (at your option) any later version.
 *
 *  RetroArch is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE.  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with RetroArch.
 *  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __RARCH_LAKKA_H
#define __RARCH_LAKKA_H

#define LAKKA_SSH_PATH       "/storage/.cache/services/sshd.conf"
#define LAKKA_SAMBA_PATH     "/storage/.cache/services/samba.conf"
#define LAKKA_BLUETOOTH_PATH "/storage/.cache/services/bluez.conf"
#define LAKKA_UPDATE_DIR     "/storage/.update/"
#define LAKKA_CONNMAN_DIR    "/storage/.cache/connman/"
#define LAKKA_LOCALAP_PATH   "/storage/.cache/services/localap.conf"
#define LAKKA_TIMEZONE_PATH  "/storage/.cache/timezone"

#define DEFAULT_TIMEZONE "UTC"
#define TIMEZONE_LENGTH 255

#endif
