
#define IDR_MENU                                101
#define IDD_PICKCORE                            103
#define ID_M_LOAD_CONTENT                       40001
#define ID_CORELISTBOX                          40002
#define ID_M_RESET                              40002
#define ID_M_QUIT                               40003
#define ID_M_MENU_TOGGLE                        40004
#define ID_M_PAUSE_TOGGLE                       40005
#define ID_M_LOAD_CORE                          40006
#define ID_M_LOAD_STATE                         40007
#define ID_M_SAVE_STATE                         40008
#define ID_M_DISK_CYCLE                         40009
#define ID_M_DISK_NEXT                          40010
#define ID_M_DISK_PREV                          40011
#define ID_M_WINDOW_SCALE_1X                    40012
#define ID_M_WINDOW_SCALE_2X                    40013
#define ID_M_WINDOW_SCALE_3X                    40014
#define ID_M_WINDOW_SCALE_4X                    40015
#define ID_M_WINDOW_SCALE_5X                    40016
#define ID_M_WINDOW_SCALE_6X                    40017
#define ID_M_WINDOW_SCALE_7X                    40018
#define ID_M_WINDOW_SCALE_8X                    40019
#define ID_M_WINDOW_SCALE_9X                    40020
#define ID_M_WINDOW_SCALE_10X                   40021
#define ID_M_FULL_SCREEN                        40022
#define ID_M_MOUSE_GRAB                         40023
#define ID_M_STATE_INDEX_AUTO                   40024
#define ID_M_TAKE_SCREENSHOT                    40025
#define ID_M_MUTE_TOGGLE                        40026
#define ID_M_TOGGLE_DESKTOP                     40027
#define IDR_ACCELERATOR1                104
