make -C libs/ilclient clean
make -C libs/vgfont clean
make -C hello_world clean
make -C hello_triangle clean
make -C hello_triangle2 clean
make -C hello_video clean
make -C hello_audio clean
make -C hello_font clean
make -C hello_dispmanx clean
make -C hello_tiger clean
make -C hello_encode clean
make -C hello_jpeg clean
make -C hello_videocube clean
make -C hello_teapot clean
make -C hello_fft clean
make -C hello_mmal_encode clean

make -C libs/ilclient
make -C libs/vgfont
make -C hello_world
make -C hello_triangle
make -C hello_triangle2
make -C hello_video
make -C hello_audio
make -C hello_font
make -C hello_dispmanx
make -C hello_tiger
make -C hello_encode
make -C hello_jpeg
make -C hello_videocube
make -C hello_teapot
make -C hello_fft
make -C hello_mmal_encode
