// Menu resources (Japanese)
//
// The encoding for this file is CP932.
#pragma code_page(932)

LANGUAGE LANG_JAPANESE, SUBLANG_NEUTRAL
IDR_MENU MENU
{
    POPUP "ファイル"
    {
        MENUITEM "コアをロード...", ID_M_LOAD_CORE
        MENUITEM "コンテンツをロード...", ID_M_LOAD_CONTENT
        MENUITEM SEPARATOR
        MENUITEM "終了", ID_M_QUIT
    }
    POPUP "コマンド"
    {
        POPUP "オーディオ設定"
        {
            MENUITEM "オーディオを消音", ID_M_MUTE_TOGGLE
        }
        POPUP "ディスク操作"
        {
            MENUITEM "ディスク取り出し", ID_M_DISK_CYCLE
            MENUITEM "前のディスク", ID_M_DISK_PREV
            MENUITEM "次のディスク", ID_M_DISK_NEXT
        }
        POPUP "保存状態設定"
        {
            POPUP "状態のインデックス"
            {
                MENUITEM "-1 (自動)", ID_M_STATE_INDEX_AUTO
                MENUITEM "0", ID_M_STATE_INDEX_AUTO
                MENUITEM "1", ID_M_STATE_INDEX_AUTO
                MENUITEM "2", ID_M_STATE_INDEX_AUTO
                MENUITEM "3", ID_M_STATE_INDEX_AUTO
                MENUITEM "4", ID_M_STATE_INDEX_AUTO
                MENUITEM "5", ID_M_STATE_INDEX_AUTO
                MENUITEM "6", ID_M_STATE_INDEX_AUTO
                MENUITEM "7", ID_M_STATE_INDEX_AUTO
                MENUITEM "8", ID_M_STATE_INDEX_AUTO
                MENUITEM "9", ID_M_STATE_INDEX_AUTO
            }
            MENUITEM "保存状態をロード", ID_M_LOAD_STATE
            MENUITEM "状態保存", ID_M_SAVE_STATE
        }
        MENUITEM "リセット", ID_M_RESET
        MENUITEM "メニューに切り替え", ID_M_MENU_TOGGLE
        MENUITEM "一時停止", ID_M_PAUSE_TOGGLE
        MENUITEM "マウスグラブ切り替え", ID_M_MOUSE_GRAB
        MENUITEM "スクリーンショットを撮る", ID_M_TAKE_SCREENSHOT
    }
    POPUP "ウィンドウ"
    {
        POPUP "ウィンドウのスケール"
        {
            MENUITEM "1x", ID_M_WINDOW_SCALE_1X
            MENUITEM "2x", ID_M_WINDOW_SCALE_2X
            MENUITEM "3x", ID_M_WINDOW_SCALE_3X
            MENUITEM "4x", ID_M_WINDOW_SCALE_4X
            MENUITEM "5x", ID_M_WINDOW_SCALE_5X
            MENUITEM "6x", ID_M_WINDOW_SCALE_6X
            MENUITEM "7x", ID_M_WINDOW_SCALE_7X
            MENUITEM "8x", ID_M_WINDOW_SCALE_8X
            MENUITEM "9x", ID_M_WINDOW_SCALE_9X
            MENUITEM "10x", ID_M_WINDOW_SCALE_10X
        }
        MENUITEM "デスクトップメニューを表示", ID_M_TOGGLE_DESKTOP
        MENUITEM "排他的なフルスクリーン切り替え", ID_M_FULL_SCREEN
        // Shader dialog is disabled for now, until video_threaded issues are fixed.
        //MENUITEM "シェーダーのパラメータ", ID_M_SHADER_PARAMETERS
    }
}

//
// Dialog resources
//
LANGUAGE LANG_JAPANESE, SUBLANG_NEUTRAL
IDD_PICKCORE DIALOG 0, 0, 225, 118
STYLE DS_3DLOOK | DS_CENTER | DS_MODALFRAME | DS_SHELLFONT | WS_CAPTION | WS_VISIBLE | WS_POPUP | WS_SYSMENU
CAPTION "コア選択"
FONT 8, "Ms Shell Dlg"
{
    LISTBOX         ID_CORELISTBOX, 5, 55, 214, 60, WS_TABSTOP | WS_VSCROLL | LBS_NOINTEGRALHEIGHT | LBS_SORT | LBS_NOTIFY, WS_EX_LEFT
    DEFPUSHBUTTON   "OK", IDOK, 170, 15, 50, 14, 0, WS_EX_LEFT
    PUSHBUTTON      "キャンセル", IDCANCEL, 170, 32, 50, 14, 0, WS_EX_LEFT
    LTEXT           "コンテンツに使用するコアを選択してください。", 0, 9, 12, 160, 17, SS_LEFT, WS_EX_LEFT
}
