#pragma once

#include <wiiu/mem/baseheap.h>
#include <wiiu/mem/blockheap.h>
#include <wiiu/mem/expandedheap.h>
#include <wiiu/mem/frameheap.h>
#include <wiiu/mem/memheap.h>
#include <wiiu/mem/memlist.h>
#include <wiiu/mem/unitheap.h>
