#ifndef __COLOR_H__
#define __COLOR_H__

// luminance is stored twice, thus one of the lum. is
// redundant, but this way we can fill the screen.

#define COLOR_BLACK        (0x00800080)
#define COLOR_MAROON       (0x266A26C0)
#define COLOR_GREEN        (0x4B554B4A)
#define COLOR_OLIVE        (0x7140718A)
#define COLOR_NAVY         (0x0EC00E75)
#define COLOR_PURPLE       (0x34AA34B5)
#define COLOR_TEAL         (0x59955940)
#define COLOR_GRAY         (0x80808080)
#define COLOR_SILVER       (0xC080C080)
#define COLOR_RED          (0x4C544CFF)
#define COLOR_LIME         (0x952B9515)
#define COLOR_YELLOW       (0xE100E194)
#define COLOR_BLUE         (0x1DFF1D6B)
#define COLOR_FUCHSIA      (0x69D469EA)
#define COLOR_AQUA         (0xB2ABB200)
#define COLOR_WHITE        (0xFF80FF80)
#define COLOR_MONEYGREEN   (0xD076D074)
#define COLOR_SKYBLUE      (0xC399C36A)
#define COLOR_CREAM        (0xFA79FA82)
#define COLOR_MEDGRAY      (0xA082A07F)

#endif /* COLOR_H */
