/* Arabic */
#define LANGUAGE_PROGRESS_ARABIC_TRANSLATED 31
#define LANGUAGE_PROGRESS_ARABIC_APPROVED   0

/* Asturian */
#define LANGUAGE_PROGRESS_ASTURIAN_TRANSLATED 14
#define LANGUAGE_PROGRESS_ASTURIAN_APPROVED   11

/* Catalan */
#define LANGUAGE_PROGRESS_CATALAN_TRANSLATED 27
#define LANGUAGE_PROGRESS_CATALAN_APPROVED   0

/* Czech */
#define LANGUAGE_PROGRESS_CZECH_TRANSLATED 98
#define LANGUAGE_PROGRESS_CZECH_APPROVED   0

/* Welsh */
#define LANGUAGE_PROGRESS_WELSH_TRANSLATED 6
#define LANGUAGE_PROGRESS_WELSH_APPROVED   0

/* Danish */
#define LANGUAGE_PROGRESS_DANISH_TRANSLATED 3
#define LANGUAGE_PROGRESS_DANISH_APPROVED   0

/* German */
#define LANGUAGE_PROGRESS_GERMAN_TRANSLATED 100
#define LANGUAGE_PROGRESS_GERMAN_APPROVED   17

/* Greek */
#define LANGUAGE_PROGRESS_GREEK_TRANSLATED 15
#define LANGUAGE_PROGRESS_GREEK_APPROVED   0

/* English, United Kingdom */
#define LANGUAGE_PROGRESS_ENGLISH_UNITED_KINGDOM_TRANSLATED 97
#define LANGUAGE_PROGRESS_ENGLISH_UNITED_KINGDOM_APPROVED   0

/* Esperanto */
#define LANGUAGE_PROGRESS_ESPERANTO_TRANSLATED 1
#define LANGUAGE_PROGRESS_ESPERANTO_APPROVED   0

/* Spanish */
#define LANGUAGE_PROGRESS_SPANISH_TRANSLATED 100
#define LANGUAGE_PROGRESS_SPANISH_APPROVED   98

/* Persian */
#define LANGUAGE_PROGRESS_PERSIAN_TRANSLATED 7
#define LANGUAGE_PROGRESS_PERSIAN_APPROVED   0

/* Finnish */
#define LANGUAGE_PROGRESS_FINNISH_TRANSLATED 64
#define LANGUAGE_PROGRESS_FINNISH_APPROVED   34

/* French */
#define LANGUAGE_PROGRESS_FRENCH_TRANSLATED 99
#define LANGUAGE_PROGRESS_FRENCH_APPROVED   99

/* Galician */
#define LANGUAGE_PROGRESS_GALICIAN_TRANSLATED 11
#define LANGUAGE_PROGRESS_GALICIAN_APPROVED   0

/* Hebrew */
#define LANGUAGE_PROGRESS_HEBREW_TRANSLATED 0
#define LANGUAGE_PROGRESS_HEBREW_APPROVED   0

/* Croatian */
#define LANGUAGE_PROGRESS_CROATIAN_TRANSLATED 1
#define LANGUAGE_PROGRESS_CROATIAN_APPROVED   0

/* Hungarian */
#define LANGUAGE_PROGRESS_HUNGARIAN_TRANSLATED 95
#define LANGUAGE_PROGRESS_HUNGARIAN_APPROVED   0

/* Indonesian */
#define LANGUAGE_PROGRESS_INDONESIAN_TRANSLATED 15
#define LANGUAGE_PROGRESS_INDONESIAN_APPROVED   0

/* Italian */
#define LANGUAGE_PROGRESS_ITALIAN_TRANSLATED 99
#define LANGUAGE_PROGRESS_ITALIAN_APPROVED   0

/* Japanese */
#define LANGUAGE_PROGRESS_JAPANESE_TRANSLATED 46
#define LANGUAGE_PROGRESS_JAPANESE_APPROVED   0

/* Korean */
#define LANGUAGE_PROGRESS_KOREAN_TRANSLATED 90
#define LANGUAGE_PROGRESS_KOREAN_APPROVED   0

/* Maltese */
#define LANGUAGE_PROGRESS_MALTESE_TRANSLATED 0
#define LANGUAGE_PROGRESS_MALTESE_APPROVED   0

/* Dutch */
#define LANGUAGE_PROGRESS_DUTCH_TRANSLATED 21
#define LANGUAGE_PROGRESS_DUTCH_APPROVED   0

/* Norwegian */
#define LANGUAGE_PROGRESS_NORWEGIAN_TRANSLATED 6
#define LANGUAGE_PROGRESS_NORWEGIAN_APPROVED   0

/* Occitan */
#define LANGUAGE_PROGRESS_OCCITAN_TRANSLATED 0
#define LANGUAGE_PROGRESS_OCCITAN_APPROVED   0

/* Polish */
#define LANGUAGE_PROGRESS_POLISH_TRANSLATED 60
#define LANGUAGE_PROGRESS_POLISH_APPROVED   16

/* Portuguese, Brazilian */
#define LANGUAGE_PROGRESS_PORTUGUESE_BRAZILIAN_TRANSLATED 98
#define LANGUAGE_PROGRESS_PORTUGUESE_BRAZILIAN_APPROVED   4

/* Portuguese */
#define LANGUAGE_PROGRESS_PORTUGUESE_TRANSLATED 15
#define LANGUAGE_PROGRESS_PORTUGUESE_APPROVED   0

/* Russian */
#define LANGUAGE_PROGRESS_RUSSIAN_TRANSLATED 86
#define LANGUAGE_PROGRESS_RUSSIAN_APPROVED   13

/* Sinhala */
#define LANGUAGE_PROGRESS_SINHALA_TRANSLATED 0
#define LANGUAGE_PROGRESS_SINHALA_APPROVED   0

/* Slovak */
#define LANGUAGE_PROGRESS_SLOVAK_TRANSLATED 13
#define LANGUAGE_PROGRESS_SLOVAK_APPROVED   0

/* Serbian (Latin) */
#define LANGUAGE_PROGRESS_SERBIAN_(LATIN)_TRANSLATED 5
#define LANGUAGE_PROGRESS_SERBIAN_(LATIN)_APPROVED   0

/* Swedish */
#define LANGUAGE_PROGRESS_SWEDISH_TRANSLATED 15
#define LANGUAGE_PROGRESS_SWEDISH_APPROVED   0

/* Turkish */
#define LANGUAGE_PROGRESS_TURKISH_TRANSLATED 99
#define LANGUAGE_PROGRESS_TURKISH_APPROVED   99

/* Ukrainian */
#define LANGUAGE_PROGRESS_UKRAINIAN_TRANSLATED 33
#define LANGUAGE_PROGRESS_UKRAINIAN_APPROVED   8

/* Valencian */
#define LANGUAGE_PROGRESS_VALENCIAN_TRANSLATED 12
#define LANGUAGE_PROGRESS_VALENCIAN_APPROVED   0

/* Vietnamese */
#define LANGUAGE_PROGRESS_VIETNAMESE_TRANSLATED 8
#define LANGUAGE_PROGRESS_VIETNAMESE_APPROVED   0

/* Chinese Simplified */
#define LANGUAGE_PROGRESS_CHINESE_SIMPLIFIED_TRANSLATED 57
#define LANGUAGE_PROGRESS_CHINESE_SIMPLIFIED_APPROVED   30

/* Chinese Traditional */
#define LANGUAGE_PROGRESS_CHINESE_TRADITIONAL_TRANSLATED 63
#define LANGUAGE_PROGRESS_CHINESE_TRADITIONAL_APPROVED   55

